[Setting category="Menu" name="Show Author"]
bool showAuthor = true;

[Setting category="Menu" name="Show Gold"]
bool showGold = false;

[Setting category="Menu" name="Show Silver"]
bool showSilver = false;

[Setting category="Menu" name="Show Bronze"]
bool showBronze = false;

[Setting category="Menu" name="Reverse Medal Order"]
bool reverseMedalOrder = false;

[Setting category="Menu" name="Hide medal if beaten"]
bool hideIfBeaten = true;

[Setting category="Next Medal" name="Next Medal for Author" description="Enables the \"Next Medal\" feature for Author Medal"]
bool nextMedalForAuthor = true;

[Setting category="Next Medal" name="Hide medal if already shown in Next Medal"]
bool hideIfNext = true;

[Setting category="Next Medal" name="Show Delta in Next Medal"]
bool showDelta = true;

enum DeltaColor
{
	Black = 0,
	Red = 1,
	RedYellowGreen = 2
}
[Setting category="Misc" name="Delta text color"]
DeltaColor deltaColor = DeltaColor::RedYellowGreen;

[Setting category="Misc" name="Shadows" description="Couldn't decide if it looks better with or without shadows, so it's up to you."]
bool shadows = true;

//rendering vars
nvg::Font nvgFont;
array<nvg::Texture@> nvgMedals;

const auto ogCenterCoords = vec2(620, 183);
const auto ogCenterCoordsDelta = vec2(680, 340);
const int ogFontSize = 35;
const int ogFontSizeDelta = 30;
const float ogMedalSize = 32.0;
const float ogMedalPadding = 3.0;

vec2 coords;
vec2 coordsDelta;

int fontSize;
int fontSizeDelta;
float textHeight;
float textWidth;
const float skewAngle = -.21;

float medalSize;
float medalPadding;
float spacing;

vec2 screenSize;

//states of ML pages refresh
bool campaignRefreshNeeded = true;
bool totdRefreshNeeded = true;

//patch data
const string patchedDAName = "patchedcustommedalinfo";
const int nbInfos = 5; //the number of infos the ML pages give us

void Main(){
	//we start by patching the ML layers
    auto app = cast<CTrackMania@>(GetApp());
	while(app.MenuManager is null 
		|| app.MenuManager.MenuCustom_CurrentManiaApp is null 
		|| app.MenuManager.MenuCustom_CurrentManiaApp.UILayers.Length <40)
		yield();
	
    auto uiLayers = app.MenuManager.MenuCustom_CurrentManiaApp.UILayers;
	for (uint i = 0; i < uiLayers.Length; i++){
		CGameUILayer@ uiLayer = uiLayers[i];
		string header = uiLayer.ManialinkPageUtf8.SubStr(0, 100);

		if (header.IndexOf("\"Page_CampaignDisplay\"") != -1){
			if(PatchCampaignLayer(uiLayer) == -1)
				print("Campaign patch failed!");
		}
		
		if (header.IndexOf("\"Page_MonthlyCampaignDisplay\"") != -1){
			if(PatchTotdLayer(uiLayer) == -1)
				print("Totd patch failed!");
		}
	}

	//Scale to correct res
	screenSize = vec2(Draw::GetWidth(), Draw::GetHeight());
	InitLengths();

	//Font and Medals loading
	nvgFont = nvg::LoadFont("DroidSans-Bold.ttf");
    nvgMedals.InsertLast(nvg::LoadTexture("medal_author_32.png", nvg::TextureFlags::GenerateMipmaps));
    nvgMedals.InsertLast(nvg::LoadTexture("medal_gold_32.png", nvg::TextureFlags::GenerateMipmaps));
    nvgMedals.InsertLast(nvg::LoadTexture("medal_silver_32.png", nvg::TextureFlags::GenerateMipmaps));
    nvgMedals.InsertLast(nvg::LoadTexture("medal_bronze_32.png", nvg::TextureFlags::GenerateMipmaps));
	
}

void InitLengths(){
    float w = Draw::GetWidth();
    float h = Draw::GetHeight();
	float xs = w/1920.0;
	float ys = h/1080.0;
    if(w/h != 16.0/9.0){
    	ys *= (w/h)/(16.0/9.0); //magic
    }
    vec2 scaling = vec2(xs,ys);
    vec2 newCenter = vec2(w/2,h/2);

    coords = ogCenterCoords*scaling + newCenter;
    coordsDelta = ogCenterCoordsDelta*scaling + newCenter;
	medalSize = ogMedalSize*xs;
	medalPadding = ogMedalPadding*xs;

	//text sizes 
	fontSize = int(ogFontSize*ys);
	fontSizeDelta = int(ogFontSizeDelta*ys);
	nvg::FontFace(nvgFont);
	nvg::FontSize(fontSize);
	textWidth = nvg::TextBounds("0:00.000").x;
	textHeight = 0.72*fontSize;
	spacing = Math::Max(textHeight,medalSize)*1.1;
}

//returns index of patch if success, -1 otherwise
int PatchLayer(CGameUILayer@ uiLayer, const string &in mapInfoName){
	//we check that the layer is not already patched
	if(uiLayer.ManialinkPageUtf8.IndexOf(patchedDAName) != -1)
		return -1;
	
	//we patch the ML page to add our custom info as a data attribute of the frame_maprankings
	//used to be TMNext_MapRankings::UpdateMapRanking
	int index = uiLayer.ManialinkPageUtf8.IndexOf("Trackmania_MapRankings::UpdateMapRanking");
	if(index == -1)
		return -1;
	uiLayer.ManialinkPageUtf8 = 
		uiLayer.ManialinkPageUtf8.SubStr(0, index) 
		+ "_State.Controls.Frame_MapRankings.DataAttributeSet(\""+patchedDAName+
		"\",\"\"^"+mapInfoName+".Uid^\",\"^"+mapInfoName+".AuthorTime^\",\"^"+
		mapInfoName+".GoldTime^\",\"^"+mapInfoName+".SilverTime^\",\"^"+mapInfoName+".BronzeTime);\n"
		+ uiLayer.ManialinkPageUtf8.SubStr(index,uiLayer.ManialinkPageUtf8.Length-index);
	return index;
}

int PatchTotdLayer(CGameUILayer@ uiLayer){
	return PatchLayer(uiLayer,"_DailyMap.Map");
}

int PatchCampaignLayer(CGameUILayer@ uiLayer){
	return PatchLayer(uiLayer,"MapData");
}

//Triggers refresh of the ML page. If we don't do this, the page only loads properly on the second render ILOVEMANIALINKPAGES
void RefreshPage(CTrackMania@ app, CGameUILayer@ uiLayer){
	MwFastBuffer<wstring> Data;
	Data.Add("Router_EnteringRoute");
	app.MenuManager.MenuCustom_CurrentManiaApp.LayerCustomEvent(@uiLayer,"Router_AddPendingEvent",Data);
}

void Render(){
	//most of the UI checks here come from NaNInf's ChampionMedals
    auto app = cast<CTrackMania@>(GetApp());
	if (app.Network.ClientManiaAppPlayground !is null //we are playing
        || app.MenuManager.MenuCustom_CurrentManiaApp is null //probably never null
		|| (app.LoadProgress !is null && app.LoadProgress.State != NGameLoadProgress::EState::Disabled)) //loading in progress
		return;
	
	//we first check if Campaign or Totd page is visible
    auto uiLayers = app.MenuManager.MenuCustom_CurrentManiaApp.UILayers;
	CGameUILayer@ mainUILayer;
	CGameUILayer@ reportLayer;
	for (uint i = 0; i < uiLayers.Length; i++){
		CGameUILayer@ uiLayer = uiLayers[i];
		if(!uiLayer.IsVisible) 
			continue;

		string header = uiLayer.ManialinkPageUtf8.SubStr(0, 100);
		if (header.IndexOf("\"Overlay_ReportSystem\"") != -1){ //Report page
			@reportLayer = uiLayer;
		}
		if (header.IndexOf("\"Page_CampaignDisplay\"") != -1){
			@mainUILayer = uiLayer;
			if(campaignRefreshNeeded){ //ML page needs refresh upon first display
				RefreshPage(app,uiLayer);
				campaignRefreshNeeded = false;
			}
			break;
		}
		if (header.IndexOf("\"Page_MonthlyCampaignDisplay\"") != -1){
			@mainUILayer = uiLayer;
			if(totdRefreshNeeded){ //ML page needs refresh upon first display
				RefreshPage(app,uiLayer);
				totdRefreshNeeded = false;
			}
			break;
		}
	}
	if(mainUILayer is null)
		return;

	if(reportLayer !is null){
		auto reportFrame = cast<CGameManialinkFrame@>(reportLayer.LocalPage.GetFirstChild("frame-report-system"));
		if(reportFrame !is null && reportFrame.Visible)
			return; //report dialog is open, so hide medals
	}

	auto profileFrame = cast<CGameManialinkFrame@>(mainUILayer.LocalPage.GetFirstChild("frame-profile"));
	if(profileFrame !is null && profileFrame.Visible)
		return; //showing profile details, so hide medals

	// we don't display medals if track is locked
	auto locked = cast<CGameManialinkQuad@>(mainUILayer.LocalPage.GetFirstChild("trackmania-map-rankings_quad-lock"));
	if(locked !is null && locked.Visible)
		return;
	
	//we check if the frame with player's time is visible (hidden if we're in records tab, etc...)
	auto medalFrame = cast<CGameManialinkFrame@>(mainUILayer.LocalPage.GetFirstChild("trackmania-map-rankings_frame-medal"));
	if(medalFrame is null || !medalFrame.Visible) 
		return;
	
	//we get the frame where the ML page stored our info
	auto mapRankingsFrame = cast<CGameManialinkFrame@>(mainUILayer.LocalPage.GetFirstChild("frame-map-rankings"));
	if(mapRankingsFrame is null)
		return;

	auto infos = mapRankingsFrame.DataAttributeGet(patchedDAName).Split(',');
	if(infos.Length != nbInfos || infos[0] =="")
		return;

	//Rescale to correct res if changed
	if (screenSize.x != Draw::GetWidth() || screenSize.y != Draw::GetHeight()){
		screenSize = vec2(Draw::GetWidth(), Draw::GetHeight());
		InitLengths();
		trace("Rescaling lengths");
	}

	string timeStr = cast<CGameManialinkLabel@>(medalFrame.GetFirstChild("trackmania-map-rankings_label-player-time")).Value;
	//if no time is set, we draw a translucid background behind the medals for better reading
	bool noTime = timeStr == "";

	// setting next medal
	int nextMedal = -1; //-1 = unset, 0 = bronze, 1 = silver, 2 = gold, 3 = AT, 10 = all
	auto nextMedalFrame = cast<CGameManialinkFrame@>(medalFrame.GetFirstChild("trackmania-map-rankings_frame-next-medal"));
	if(!noTime){
		if(nextMedalFrame.Visible){
			string img = cast<CGameManialinkQuad@>(nextMedalFrame.GetFirstChild("trackmania-map-rankings_quad-medal")).ImageUrl;
			nextMedal = Text::ParseInt(img.SubStr(img.IndexOf("_Medal_")-1,1));
		}
		else{
			if(UnFormatTime(timeStr) <= Text::ParseInt(infos[1])){
				nextMedal = 10;
			}
			else{
				nextMedal = 3;
			}
		}
	}
	
	// next medal for AT
	if(nextMedal == 3){
		if(nextMedalForAuthor){
			nextMedalFrame.Visible = true;
			cast<CGameManialinkQuad@>(nextMedalFrame.GetFirstChild("trackmania-map-rankings_quad-medal")).ImageUrl = "file://Media/Manialinks/Nadeo/Trackmania/Menus/Medals/3_Medal_Nadeo_Small.dds";
			cast<CGameManialinkLabel@>(nextMedalFrame.GetFirstChild("trackmania-map-rankings_label-medal-time")).Value = FormatTime(infos[1]);
		} else
			nextMedalFrame.Visible = false;
	}
	
	// select medals to draw
	int[] medals;
	bool[] medalsToShow = {showBronze, showSilver, showGold, showAuthor};
	for(int i = 3; i >= 0; i--){
		if(medalsToShow[i] && (!hideIfBeaten || nextMedal <= i) && (!nextMedalFrame.Visible || !hideIfNext || nextMedal != i))
			medals.InsertLast(3-i);
	}
	if(reverseMedalOrder)
		medals.Reverse();
		
	for(uint i = 0; i < medals.Length; i++){
		DrawMedal(FormatTime(infos[medals[i]+1]), nvgMedals[medals[i]], i, noTime);
	}
	
	// Delta 
	if(showDelta && nextMedal >= 0 && (nextMedal <=2 || (nextMedal ==3 && nextMedalForAuthor)))
		DrawDelta(UnFormatTime(timeStr) - Text::ParseInt(infos[4-nextMedal]));
	
}

string FormatTime(const string &in timeString){
	return Time::Format(Text::ParseInt(timeString));
}

int UnFormatTime(const string &in formatedString){
	return Time::ParseRelativeTime(formatedString);
}

//this part is inspired by Miss' LoadProgress and NaNInf's ChampionMedals
void DrawMedal(const string &in str, nvg::Texture@ medal, int medalIndex = 0, bool drawBackground = false){
	vec2 totalBounds = vec2(medalSize + medalPadding + textWidth, Math::Max(medalSize, textHeight));
	nvg::Translate(coords + vec2(-5*medalIndex,medalIndex*spacing) -0.5*totalBounds);
	
	//draw shadowed background for better reading
	if(drawBackground){
		nvg::BeginPath();
		vec2 bgBounds = totalBounds + vec2(nvg::TextBounds(str).x - textWidth, 0);
		nvg::FillPaint(nvg::BoxGradient(.1*bgBounds, .9*bgBounds, 5.0, 30.0,  vec4(0, 0, 0, .7), vec4(0, 0, 0, 0)));
		nvg::Fill();
		nvg::ClosePath();
	}

	//draw medal
	vec2 medalPos = vec2(0,totalBounds.y/2.0-medalSize/2.0);
	vec2 medalBounds = vec2(medalSize,medalSize);
	
	if(shadows){
		nvg::BeginPath();
		nvg::FillPaint(nvg::RadialGradient(
			medalPos+medalBounds/2.0 +vec2(3,3), 
			(0.5 * medalSize)/2.0, (0.95 * medalSize)/2.0, vec4(0, 0, 0, 1), vec4(0, 0, 0, 0)));
		nvg::Fill();
		nvg::ClosePath();
	}

	nvg::BeginPath();
	nvg::FillPaint(nvg::TexturePattern(medalPos, medalBounds, 0, medal, 1.0));
	nvg::Rect(medalPos, medalBounds);
	nvg::Fill();
	nvg::ClosePath();
	
	//draw text
	nvg::BeginPath();
	nvg::FontFace(nvgFont);
	nvg::FontSize(fontSize);
	
	nvg::Translate(vec2(medalSize+3, textHeight + totalBounds.y/2.0 -textHeight/2.0));
	nvg::SkewX(skewAngle);
		
	if(shadows){
		nvg::FillColor(vec4(0, 0, 0, 1));
		nvg::FontBlur(3.0);
		nvg::Text(vec2(3, 3), str);
	}

	nvg::FillColor(vec4(1, 1, 1, 1));
	nvg::FontBlur(0.0);
	nvg::Text(vec2(0,0), str);
	
	nvg::ClosePath();
	nvg::ResetTransform();
}

void DrawDelta(int delta){
	string str = "+" + Time::Format(delta);
	
	nvg::BeginPath();
	nvg::FontFace(nvgFont);
	nvg::FontSize(fontSizeDelta);
	nvg::Translate(coordsDelta);
	nvg::SkewX(skewAngle);
	
	if(shadows){
		nvg::FillColor(vec4(0, 0, 0, 1));
		nvg::FontBlur(1.0);
		nvg::Text(vec2(1, 1), str);
	}
	
	nvg::FillColor(vec4(0, 0, 0, .8));
	nvg::FontBlur(0.0);
	nvg::Text(vec2(1, 1), str);
		
	// color depending on score
	vec4 color = vec4(.85, .03, .18, 1);
	if(deltaColor == DeltaColor::RedYellowGreen){
		if(delta <= 500 && delta > 100)
			color = vec4(.9, .4, .13, 1);
		else if(delta <= 100)
			color = vec4(0, .50, .31, 1);
	} 
	else if(deltaColor == DeltaColor::Black){
		color = vec4(.01,.01,.01,1);
	}
	
	nvg::FillColor(color);
	nvg::FontBlur(0.0);
	nvg::Text(vec2(0,0), str);
	nvg::ClosePath();
}


void OnSettingsChanged(){
	if(showAuthor && showGold && showSilver && showBronze && !hideIfNext)
		UI::ShowNotification("Warning: Too many medals",
		"Displaying 4 medals conflicts with \"Next Medal\" message.",
		vec4(0.4, 0,0, 1),
		2000);
}
